#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "libetm.h"

int big_error(int error_code, const char *str1, const char *str2, const char *str3, const char *str4)
{
	char	error_str[64 * 5 + 1];

	if (error_code <= LIB_LASTERRORCODE) {
		/*
		 * str1 = error location (func) or ? -> not sure...
		 * well, this is f------ messy
		 */
		snprintf(error_str, 64 * 5 + 1, "FATAL ERROR: Libetm: %s: %s - Will quit now",
			str1, get_lib_error_str(error_code));
	}
	else {
		str_n_cpy(error_str, "FATAL ERROR: Application:", 32);
		if (str1[0] != '\0') {
			str_n_cat(error_str, " ", 1);
			str_n_cat(error_str, str1, 63);
		}
		if (str2[0] != '\0') {
			str_n_cat(error_str, " ", 1);
			str_n_cat(error_str, str2, 63);
		}
		if (str3[0] != '\0') {
			str_n_cat(error_str, " ", 1);
			str_n_cat(error_str, str3, 63);
		}
		if (str4[0] != '\0') {
			str_n_cat(error_str, " ", 1);
			str_n_cat(error_str, str4, 63);
		}
		str_n_cat(error_str, " - Will quit now", 32);
	}
	fprintf(stderr, "%s\n", error_str);
	exit(error_code);
	return error_code;
}

int main(int argc, char *argv[])
{
	char	*str1, *str2, str3[512], str4[512];
	char	msg[128];
	int	i, c;
	double	size, tmp;
	time_t	time1, time2;
	unsigned long j;

	printf("\n=== testing libetm ===\n\n");
	printf("version:\n%s - %s\n%s\n\n",
				lib_version(0), lib_version(1), lib_version(2));

	printf(	"first, some sizeof() stuff:\n"
		"  sizeof(int) = %d - sizeof(long) = %d - sizeof(size_t) = %d\n"
		"  sizeof(char) = %d - sizeof(char *) = %d\n\n",
		sizeof(int), sizeof(long), sizeof(size_t), sizeof(char), sizeof(char *));

	printf("itoa2() testing\n");
	if (question("next step"))
		for (j = 0; j < 1000 ; j += 1)
			printf("%s\n", itoa2(j));
	if (question("next step"))
		for (j = 1000; j < 100000 ; j += 1)
			printf("%s\n", itoa2(j));
	if (question("next step - j+= 87"))
		for (j = 100000; j <= 10000000 + 87; j += 87)
			printf("\r%s", itoa2(j));
	printf("\n");
	if (question("next step - j += 3471"))
		for (j = 0; j <= 1000000000L + 3471; j += 3471)
			printf("\r%s", itoa2(j));
	printf("\n");

	if (question("test #1: checking 25000 calls to l_str_cat(str, short_str)")) {
		printf("please wait...\n");

		str1 = l_str_new(NULL);
		time1 = time(NULL);
		for (i = 0; i < 25000; i++) {
			snprintf(str3, 512, "#%d-", i);
			str1 = l_str_cat(str1, str3);
		}
		time2 = time(NULL);
		printf("\n%s\n", str1);
		l_str_free(str1);
		printf("done in %.0f s\n\n", difftime(time2, time1));
	}

	if (question("test #2-0: checking readable_size(size)")) {
		for (size = 0; size <= (double)25000 * 1000000; size += 2850001)
			printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		printf("done\n\n");
	}

	if (question("test #2-1: checking readable_size(size)")) {
		size = 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 1024 + 512;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 1024 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 1024 * 1024 + 800 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 1024 * 1024 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 1024 * 1024 * 1024 + 512 * 1024 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 3 * 512 * 1024 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));
		size = 7 * 256 * 1024 * 1024;
		printf("readable_size(%.0f) = %s\n", size, readable_size(size));

		printf("\ncomparing to results from nautilus:\n");
		size = 18683;
		printf("readable_size(%.0f) = %s (=  18.2 KB?)\n", size, readable_size(size));
		size = 440791;
		printf("readable_size(%.0f) = %s (=  430.5 KB?)\n", size, readable_size(size));
		size = 20662806;
		printf("readable_size(%.0f) = %s (=  19.7 MB?)\n", size, readable_size(size));

		printf("\nmore checking with results expected by us this time:\n");
		size = 1024 * 1024 + 512 * 1024 - 1;
		printf("readable_size(%.0f) = %s (= 1.4 MB?)\n", size, readable_size(size));
		size = 1024 * 1024 + 512 * 1024;
		printf("readable_size(%.0f) = %s (= 1.5 MB?)\n", size, readable_size(size));

		printf("done\n\n");
	}

	if (question("test #2-2:\n")) {
		printf("size1 = readable_size(12345678L) = %s\n", readable_size(12345678L));
		printf("size2 = readable_size(1234567890L) = %s\n", readable_size(1234567890L));
		printf("checking: printf('$s / $s', readable_size(12345678L), readable_size(1234567890L))\n"
			"-> 2 calls to readable_size() from 1 call to printf()\n");
		printf("-> sth wrong if size1 == size2 \n");
		printf("\n%s should be != %s\n\n",
			readable_size(12345678L), readable_size(1234567890L));
		/*
		printf("\n%s should be != ", readable_size(12345678L));
		printf("%s\n\n", readable_size(1234567890L));
		*/
		printf("done\n\n");
	}

	if (question("test #2-3:\n")) {
		for (j = 0; j <= 2 * 1024; j++)
			printf("readable_size(%ld x %d x %d) = %s\n", j, 1024, 1024,
				readable_size(j * 1024 * 1024));
		printf("done\n\n");
	}

	if (question("test #3: checking malloc2(size)")) {
		if (1) {
			for (size = 1; size <= (double)200 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)300 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)400 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)500 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)600 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)1000 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
		if (question("next step")) {
			for (; size <= (double)10000 * 1000000; size += 35001) {
				printf("malloc2(%s) - ", readable_size(size));
				str1 = malloc2((size_t)size);
				free2(str1);
				printf("ok\n");
			}
			printf("done\n\n");
		}
	}

	printf("WARNING: you're entering overload area which is likely to hang up your machine\n");
	if (question("test #4: more l_str_cat(bigger, bigger) checking")) {
		size = 0;
		str1 = l_str_new("        ");
		while (size < (double) 100000) {
			str2 = l_str_new(str1);
			str1 = l_str_cat(str1, str2);
			size = strlen(str1);
			printf("l_str_cat(%s, %s) ok\n", readable_size(size),
				readable_size(size));
			l_str_free(str2);
		}
		while (question("next step")) {
			str2 = l_str_new(str1);
			time1 = time(NULL);
			str1 = l_str_cat(str1, str2);
			time2 = time(NULL);
			size = strlen(str1);
			printf("l_str_cat(%s, %s) ok in %.0f s\n", readable_size(size),
				readable_size(size), difftime(time2, time1));
			l_str_free(str2);
		}
		l_str_free(str1);
		printf("done\n\n");
	}

	if (question("DON'T!!!! test #5: checking error handling")) {
		if (question("calling: malloc2((size_t)1000000 * (size_t)1000000)")) {
			str1 = malloc2((size_t)1000000 * (size_t)1000000);
			free2(str1);
		}

		if (question("calling: realloc2((size_t)1000000 * (size_t)1000000")) {
			str1 = malloc2((size_t)1);
			str1 = realloc2(str1, (size_t)1000000 * (size_t)1000000);
			free2(str1);
		}
	}

	if (question("test #6: calling l_str_cat(str1, str1)")) {
		str1 = l_str_new("hello");
		str2 = l_str_new("hello");
		for (i = 0; i < 10; i ++) {
			str1 = l_str_cat(str1, str2);
			l_str_free(str2);
			str2 = l_str_new(str1);
		}
		l_str_free(str2);
		printf("str1 = '%s'\n", str1);
		snprintf(str3, 512, "calling l_str_cat(str1, str1)");
		for (;;) {
			printf("str1 size = '%s'\n", readable_size(strlen(str1)));
			if (!question(str3))
				break;
			str1 = l_str_cat(str1, str1);
			if (question("printing out str1")) {
				printf("str1 = '%s'\n", str1);
			}
		}
		free2(str1);
		printf("done\n\n");
	}

	if (question("test #7: str_n_cat(str3, str3)")) {
		str_n_cpy(str3, "This is str3", 64);
		if (question(str3)) {
			str_n_cat(str3, str3, 64);
			printf("%s\n", str3);
		}
	}

	if (question("now what?")) {
		printf("hmmm... that's it!\n");
	}
	return 0;
}

